@@ -6,7 +6,7 @@
 ;; Maintainer: Fontaine Development <~protesilaos/fontaine@lists.sr.ht>
 ;; URL: https://git.sr.ht/~protesilaos/fontaine
 ;; Mailing-List: https://lists.sr.ht/~protesilaos/fontaine
-;; Version: 0.3.0
+;; Version: 0.4.0
 ;; Package-Requires: ((emacs "27.1"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -139,45 +139,42 @@
 
 (defcustom fontaine-presets
   '((regular
-     :default-family "Hack"
-     :default-weight normal
-     :default-height 100
-     :fixed-pitch-family "Fira Code"
-     :fixed-pitch-weight nil ; falls back to :default-weight
-     :fixed-pitch-height 1.0
-     :variable-pitch-family "Noto Sans"
-     :variable-pitch-weight normal
-     :variable-pitch-height 1.0
-     :bold-family nil ; use whatever the underlying face has
-     :bold-weight bold
-     :italic-family "Source Code Pro"
-     :italic-slant italic
-     :line-spacing 1)
+     :default-height 100)
     (large
-     :default-family "Iosevka"
-     :default-weight normal
-     :default-height 150
+     :default-weight semilight
+     :default-height 140
+     :bold-weight extrabold)
+    (t
+     ;; I keep all properties for didactic purposes, but most can be
+     ;; omitted.
+     :default-family "Monospace"
+     :default-weight regular
+     :default-height 100
      :fixed-pitch-family nil ; falls back to :default-family
      :fixed-pitch-weight nil ; falls back to :default-weight
      :fixed-pitch-height 1.0
-     :variable-pitch-family "FiraGO"
-     :variable-pitch-weight normal
-     :variable-pitch-height 1.05
+     :fixed-pitch-serif-family nil ; falls back to :default-family
+     :fixed-pitch-serif-weight nil ; falls back to :default-weight
+     :fixed-pitch-serif-height 1.0
+     :variable-pitch-family "Sans"
+     :variable-pitch-weight nil
+     :variable-pitch-height 1.0
      :bold-family nil ; use whatever the underlying face has
      :bold-weight bold
-     :italic-family nil ; use whatever the underlying face has
+     :italic-family nil
      :italic-slant italic
-     :line-spacing 1))
+     :line-spacing nil))
   "Alist of desired typographic properties.
 
 The car of each cell is an arbitrary symbol that identifies
-and/or describes the set of properties (e.g. 'small', 'reading').
+and/or describes the set of properties (e.g. small, reading).
 
 A preset whose car is t is treated as the default option.  This
 makes it easier to specify multiple presets without duplicating
 their properties.  The other presets beside t act as overrides of
 the defaults and, as such, need only consist of the properties
-that change from the default.
+that change from the default.  See the default value of this
+variable for how that is done.
 
 The cdr is a plist that specifies the typographic properties of
 the faces `default', `fixed-pitch', `variable-pitch', `bold', and
@@ -202,6 +199,11 @@ The properties in detail:
   fallback values are `:default-family', `:default-weight', and
   1.0, respectively.
 
+- `:fixed-pitch-serif-family', `:fixed-pitch-serif-weight',
+  `:fixed-pitch-serif-height' apply to the `fixed-pitch-serif'
+  face.  Their fallback values are `:default-family',
+  `:default-weight', and 1.0, respectively.
+
 - The `:variable-pitch-family', `:variable-pitch-weight', and
   `:variable-pitch-height' apply to the `variable-pitch' face.
   They all fall back to the respective default values, as
@@ -248,7 +250,6 @@ Caveats or further notes:
 - Fontaine does not [yet] support Emacs' fontsets for other
   scripts or character sets (e.g. Emoji).  Read the documentation
   in the Info node `(emacs) Modifying Fontsets'."
-  :group 'fontaine
   :type `(alist
           :value-type
           (plist :options
@@ -260,6 +261,10 @@ Caveats or further notes:
                   ((const :tag "Fixed pitch regular weight" :fixed-pitch-weight) ,fontaine--weights-widget)
                   ((const :tag "Fixed pitch height" :fixed-pitch-height) float)
 
+                  ((const :tag "Fixed pitch serif font family" :fixed-pitch-serif-family) string)
+                  ((const :tag "Fixed pitch serif regular weight" :fixed-pitch-serif-weight) ,fontaine--weights-widget)
+                  ((const :tag "Fixed pitch serif height" :fixed-pitch-serif-height) float)
+
                   ((const :tag "Variable pitch font family" :variable-pitch-family) string)
                   ((const :tag "Variable pitch regular weight" :variable-pitch-weight) ,fontaine--weights-widget)
                   ((const :tag "Variable pitch height" :variable-pitch-height) float)
@@ -278,6 +283,8 @@ Caveats or further notes:
 
                   ((const :tag "Line spacing" :line-spacing) ,(get 'line-spacing 'custom-type))))
           :key-type symbol)
+  :package-version '(fontaine . "0.4.0")
+  :group 'fontaine
   :link '(info-link "(fontaine) Shared and implicit fallback values for presets"))
 
 (defcustom fontaine-latest-state-file
@@ -289,6 +296,7 @@ which should be assigned to a hook (e.g. `kill-emacs-hook').
 This is then used to restore the last value with the function
 `fontaine-restore-latest-preset'."
   :type 'file
+  :package-version '(fontaine . "0.1.0")
   :group 'fontaine)
 
 (defcustom fontaine-font-families nil
@@ -325,6 +333,7 @@ combine the other two lists."
           (cons :tag "Variable pitch font families"
                 (const variable-pitch)
                 (repeat string)))
+  :package-version '(fontaine . "0.2.0")
   :group 'fontaine)
 
 ;;;; General utilities
@@ -374,6 +383,10 @@ ARGS are its routines."
      (if-let ((properties (append (alist-get preset fontaine-presets)
                                   (alist-get t fontaine-presets))))
          ,args
+       ;; FIXME 2022-09-07: Because we `append' the t of
+       ;; `fontaine-presets' this error is only relevant when the list
+       ;; is empty.  Perhaps we can harden the condition.  Otherwise we
+       ;; should reword this.
        (user-error "%s is not in `fontaine-presets' or is empty" preset))))
 
 (fontaine--apply-preset
@@ -398,6 +411,16 @@ ARGS are its routines."
   (or (plist-get properties :fixed-pitch-height) 1.0)
   frame))
 
+(fontaine--apply-preset
+ fontaine--apply-fixed-pitch-serif-preset
+ "Set `fixed-pitch-serif' face attributes based on PRESET for optional FRAME."
+ (fontaine--set-face-attributes
+  'fixed-pitch-serif
+  (or (plist-get properties :fixed-pitch-family) (plist-get properties :default-family))
+  (or (plist-get properties :fixed-pitch-weight) (plist-get properties :default-weight))
+  (or (plist-get properties :fixed-pitch-height) 1.0)
+  frame))
+
 (fontaine--apply-preset
  fontaine--apply-variable-pitch-preset
  "Set `variable-pitch' face attributes based on PRESET for optional FRAME."
@@ -410,21 +433,21 @@ ARGS are its routines."
 
 (fontaine--apply-preset
  fontaine--apply-bold-preset
-  "Set `bold' face attributes based on PRESET for optional FRAME."
-  (fontaine--set-face-attributes
-   'bold
-   (or (plist-get properties :bold-family) 'unspecified)
-   (or (plist-get properties :bold-weight) 'bold)
-   (or (plist-get properties :variable-pitch-height) 1.0)
-   frame))
+ "Set `bold' face attributes based on PRESET for optional FRAME."
+ (fontaine--set-face-attributes
+  'bold
+  (or (plist-get properties :bold-family) 'unspecified)
+  (or (plist-get properties :bold-weight) 'bold)
+  (or (plist-get properties :variable-pitch-height) 1.0)
+  frame))
 
 (fontaine--apply-preset
  fontaine--apply-italic-preset
-  "Set `italic' face attributes based on PRESET for optional FRAME."
-  (fontaine--set-italic-slant
-   (or (plist-get properties :italic-family) 'unspecified)
-   (or (plist-get properties :italic-slant) 'italic)
-   frame))
+ "Set `italic' face attributes based on PRESET for optional FRAME."
+ (fontaine--set-italic-slant
+  (or (plist-get properties :italic-family) 'unspecified)
+  (or (plist-get properties :italic-slant) 'italic)
+  frame))
 
 (defvar fontaine--font-display-hist '()
   "History of inputs for display-related font associations.")
@@ -453,8 +476,10 @@ ARGS are its routines."
 (defvar fontaine-set-preset-hook nil
   "Hook that runs after setting fonts.")
 
-(defvar fontaine--current-preset nil
-  "Current font set in `fontaine-presets'.")
+(defvar fontaine-current-preset nil
+  "Current font set in `fontaine-presets'.
+This is the preset last used by `fontaine-set-preset'.  Also see
+the command `fontaine-apply-current-preset'.")
 
 ;;;###autoload
 (defun fontaine-set-preset (preset &optional frame)
@@ -470,7 +495,10 @@ frame and apply the effects to it.
 
 When called interactively with a universal prefix
 argument (\\[universal-argument]), FRAME is interpreted as
-non-nil."
+non-nil.
+
+Set `fontaine-current-preset' to PRESET.  Also see the command
+`fontaine-apply-current-preset'."
   (interactive
    (list
     (if (= (length fontaine-presets) 1)
@@ -481,17 +509,40 @@ non-nil."
       (user-error "Cannot use this in a terminal emulator; try the Emacs GUI")
     (fontaine--apply-default-preset preset frame)
     (fontaine--apply-fixed-pitch-preset preset frame)
+    (fontaine--apply-fixed-pitch-serif-preset preset frame)
     (fontaine--apply-variable-pitch-preset preset frame)
     (fontaine--apply-bold-preset preset frame)
     (fontaine--apply-italic-preset preset frame)
-    (setq fontaine--current-preset preset)
+    (setq fontaine-current-preset preset)
     (add-to-history 'fontaine--preset-history (format "%s" preset))
     (run-hooks 'fontaine-set-preset-hook)))
 
+;;;###autoload
+(defun fontaine-apply-current-preset ()
+  "Use `fontaine-set-preset' on `fontaine-current-preset'.
+The value of `fontaine-current-preset' must be one of the keys in
+`fontaine-presets'.
+
+Re-applying the current preset is useful when a new theme is
+loaded which overrides certain font families.  For example, if
+the theme defines the `bold' face without a `:family', loading
+that theme will make `bold' use the `default' family, even if the
+`fontaine-presets' are configured to have different families
+between the two.  In such a case, applying the current preset at
+the post `load-theme' phase (e.g. via a hook) ensures that font
+configurations remain consistent.
+
+Some themes that provide hooks of this sort are the
+`modus-themes' and `ef-themes' (both by Protesilaos)."
+  (interactive)
+  (when-let* ((current fontaine-current-preset)
+              ((alist-get current fontaine-presets)))
+    (fontaine-set-preset current)))
+
 ;;;; Modify individual faces
 
 (defconst fontaine--font-faces
-  '(default fixed-pitch variable-pitch bold italic)
+  '(default fixed-pitch fixed-pitch-serif variable-pitch bold italic)
   "List of faces whose typographic attributes we may change.")
 
 (defconst fontaine--font-weights
@@ -506,10 +557,6 @@ non-nil."
 (defvar fontaine--face-history '()
   "Minibuffer history of `fontaine-set-face-font'.")
 
-(define-obsolete-variable-alias
-  'fontaine--font-family-history
-  'fontaine--default-font-family-history "2022-04-29")
-
 (defvar fontaine--default-font-family-history '()
   "Minibuffer history of selected `default' font families.")
 
@@ -524,9 +571,6 @@ Target FRAME, if provided as an optional argument."
     (seq-filter
      (lambda (fam)
        (aref fam 5))
-     ;; FIXME 2022-04-26: Is `x-family-fonts' the right call here, or
-     ;; will it not work in some Emacs builds?
-     ;;
      ;; NOTE 2022-04-26: `x-family-fonts' and `x-list-fonts' accept a
      ;; pattern, but I cannot find how to use it properly to filter out
      ;; certain families.
@@ -575,8 +619,10 @@ Target FRAME, if provided as an optional argument."
 (defvar fontaine--fixed-pitch-font-family-history '()
   "Minibuffer history of selected `fixed-pitch' font families.")
 
-(defun fontaine--set-fixed-pitch (&optional frame)
-  "Set `fixed-pitch' attributes, optionally for FRAME."
+(defun fontaine--set-fixed-pitch (&optional frame serif)
+  "Set `fixed-pitch' attributes, optionally for FRAME.
+If optional SERIF is non-nil, operate on the `fixed-pitch-serif'
+face."
   (let* ((families (or (alist-get 'fixed-pitch fontaine-font-families)
                        (fontaine--family-list-fixed-pitch)))
          (family (completing-read "Font family of `fixed-pitch': "
@@ -585,9 +631,10 @@ Target FRAME, if provided as an optional argument."
          (weight (intern (completing-read "Select weight for `fixed-pitch': "
                                           fontaine--font-weights nil)))
          (height (read-number "Height of `fixed-pitch' face (must be a floating point): "
-                              1.0 'fontaine--float-history)))
+                              1.0 'fontaine--float-history))
+         (face (if serif 'fixed-pitch-serif 'fixed-pitch)))
     (if (floatp height)
-        (fontaine--set-face-attributes 'fixed-pitch family weight height frame)
+        (fontaine--set-face-attributes face family weight height frame)
       (user-error "Height of `fixed-pitch' face must be a floating point"))))
 
 (defvar fontaine--variable-pitch-font-family-history '()
@@ -665,6 +712,7 @@ non-nil."
     ('bold (fontaine--set-bold frame))
     ('default (fontaine--set-default frame))
     ('fixed-pitch (fontaine--set-fixed-pitch frame))
+    ('fixed-pitch-serif (fontaine--set-fixed-pitch frame :serif))
     ('italic (fontaine--set-italic frame))
     ('variable-pitch (fontaine--set-variable-pitch frame))
     (_ (call-interactively #'fontaine-set-preset))))
